/*
** Module   : ColorSpec.java
** Abstract : Implements Progress color-phrase details.
**
** Copyright (c) 2005-2012, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 NVS 20051026 NEW   @23274 Created initial version.
** 002 SIY 20080331 CHG   @37736 Improved support for COLOR_VALUE mode.
**                               Ref: #37732
** 003 NVS 20080420 CHG   @38047 Color value that encodes one of the regular
**                               color specifications MESSAGE/INPUT/NORMAL
**                               is not stored as value but converts into the
**                               corresponding specification right away.
** 004 SIY 20080429 CHG   @38143 Return invalid color if it is not present in
**                               color map.
** 005 SIY 20101122              Minor cleanups.
** 006 SIY 20101223              Complete rewrite due to change of color
**                               handling approach.
** 007 CS  20121031              Removed client specific references
*/

package com.goldencode.p2j.ui;

import java.io.Serializable;
import java.util.*;

/**
 * Class ColorSpec supports the color-phrase specification.
 * <ul>
 *   <li>- defines color-phrase instance
 *   <li>- NORMAL | INPUT | MESSAGES
 *   <li>- protermcap-attribute
 *   <li>- dos-hex-attribute
 * </ul>
 * The following options are NOT SUPPORTED:
 * <ul>
 *   <li>- VALUE(expression)
 *   <li>- BLINK- BRIGHT- fgnd-color bgnd-color (WINDOWS only)
 *   <li>- BLINK- RVV- UNDERLINE- BRIGHT- fgnd-color (WINDOWS only).
 * </ul>
 * <p>
 * This class does not interpret the meaning of the color value.
 *
 * @author   NVS
 * @version  2.0
 */
public class ColorSpec
implements Serializable
{
   /** COLOR_VALUE definition */
   public static final int COLOR_VALUE = 3;

   /** Color name constant for NORMAL color. */
   public static final String COLOR_NORMAL   = "NORMAL";

   /** Color name constant for INPUT color. */
   public static final String COLOR_INPUT    = "INPUT";

   /** Color name constant for MESSAGES color. */
   public static final String COLOR_MESSAGES = "MESSAGES";

   /** Color name constant for BOLD color. */
   public static final String COLOR_BOLD     = "BOLD";

   /** Color name constant for BLINK color. */
   public static final String COLOR_BLINK    = "BLINK";

   /** Color name definition value */
   private String colorValue = new String("NORMAL");

   /** Color mapping table. */
   private static Map<String, Color> colorMap = new LinkedHashMap<String, Color>();

   /* Initialize local map */
   static
   {
      colorMap.put(COLOR_NORMAL  , Color.DEFAULT);
      colorMap.put(COLOR_INPUT   , Color.DEFAULT.withUnderline());
      colorMap.put(COLOR_MESSAGES, Color.DEFAULT.withReverse());
      colorMap.put(COLOR_BOLD    , Color.DEFAULT.withBold());
      colorMap.put(COLOR_BLINK   , Color.DEFAULT.withBlink());
      //TODO: fill map from configuration?
   }

   /**
    * Default constructor.
    */
   public ColorSpec()
   {
   }

   /**
    * Copy constructor.
    *
    * @param   color
    *          existing color specification to duplicate
    */
   public ColorSpec(ColorSpec color)
   {
      this.colorValue = color.colorValue;
   }

   /**
    * Main constructor.
    *
    * @param   value
    *          color name.
    */
   public ColorSpec(String value)
   {
      String key = (value == null) ? "NORMAL" : value.toUpperCase();

      // try value as is, if not found - try same value but converted to
      // upper case and finally fall back to NORMAL
      colorValue = colorMap.containsKey(value) ? value :
                           (colorMap.containsKey(key) ? key : "NORMAL");
   }

   /**
    * Build color specification using integer value as name.
    *
    * @param   intValue
    *          Color number.
    */
   public ColorSpec(int intValue)
   {
      this(String.valueOf(intValue));
   }

   /**
    * Convert color specification into real color.
    *
    * @return  color value
    */
   public Color convert()
   {
      return colorMap.get(colorValue);
   }

   /**
    * Get array of changed entries in color mapping table.
    *
    * @param   lastIndex
    *          Starting index for the list of changed entries.
    *
    * @return  array of changed entries.
    */
   public static List<Color> getChanges(int lastIndex)
   {
      List<Color> list = new ArrayList<Color>(colorMap.values());

      return list.subList(lastIndex, list.size());
   }

   /**
    * Add new mapping from string to color.
    *
    * @param   string
    *          Color key
    * @param   color
    *          Color.
    */
   public static void addMapping(String string, Color color)
   {
      colorMap.put(string, color);
   }
}
